import { fileURLToPath } from 'url';
import { defineNuxtModule, requireModule, addTemplate, addAutoImport, addComponentsDir } from '@nuxt/kit';
import chalk from 'chalk';
import consola from 'consola';
import * as fse from 'fs-extra';
import { resolve, join } from 'pathe';
import defu from 'defu';
import { genImport } from 'knitwork';

const name = "@nuxtjs/sanity";
const version = "1.0.0-beta.0";

function validateConfig({ projectId, dataset }) {
  if (!projectId) {
    consola.warn(`Make sure you specify a ${chalk.bold("projectId")} in your sanity config.`);
    return false;
  } else {
    consola.info(`Enabled ${chalk.bold("@nuxtjs/sanity")} for project ${chalk.bold(projectId)} (${chalk.bold(dataset)}).`);
    return true;
  }
}
function getDefaultSanityConfig(jsonPath) {
  try {
    const { projectId, dataset } = fse.readJSONSync(jsonPath).api;
    return { projectId, dataset };
  } catch {
    return {};
  }
}
const CONFIG_KEY = "sanity";
const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: CONFIG_KEY,
    compatibility: {
      bridge: true
    }
  },
  defaults: (nuxt) => ({
    contentHelper: true,
    imageHelper: true,
    dataset: "production",
    apiVersion: "1",
    withCredentials: false,
    additionalClients: {},
    ...getDefaultSanityConfig(resolve(nuxt.options.rootDir, "./sanity.json"))
  }),
  async setup(options, nuxt) {
    if (!("useCdn" in options)) {
      options.useCdn = process.env.NODE_ENV === "production" && !options.token;
    }
    if (!validateConfig(options))
      return;
    try {
      if (!options.minimal) {
        options.minimal = !requireModule("@sanity/client");
      }
    } catch {
      options.minimal = true;
      consola.warn(`Not using ${chalk.bold("@sanity/client")} as it cannot be resolved in your project dependencies.
       Try running ${chalk.bold("yarn add @sanity/client")} or ${chalk.bold("npm install @sanity/client")}.
       To disable this warning, set ${chalk.bold("sanity: { minimal: true }")} in your nuxt.config.js.`);
    }
    nuxt.options.publicRuntimeConfig.sanity = defu(nuxt.options.publicRuntimeConfig.sanity, {
      useCdn: options.useCdn,
      projectId: options.projectId,
      dataset: options.dataset,
      apiVersion: options.apiVersion,
      withCredentials: options.withCredentials,
      token: options.token,
      additionalClients: options.additionalClients
    });
    const runtimeDir = fileURLToPath(new URL("./runtime", import.meta.url));
    nuxt.options.build.transpile.push(runtimeDir, "@nuxtjs/sanity");
    addTemplate({
      filename: "sanity-client.mjs",
      getContents: () => [
        options.minimal ? genImport(join(runtimeDir, "client"), ["createClient"]) : genImport("@sanity/client", "createClient"),
        "export { createClient }"
      ].join("\n")
    });
    addAutoImport([
      { name: "createClient", as: "createSanityClient", from: "#build/sanity-client.mjs" },
      { name: "groq", as: "groq", from: join(runtimeDir, "groq") },
      { name: "useSanity", as: "useSanity", from: join(runtimeDir, "composables") }
    ]);
    await addComponentsDir({
      path: join(runtimeDir, "components"),
      extensions: ["js", "ts", "mjs"]
    });
  }
});

export { module as default };
